/*******************************************************************************
* File Name: MODEMINT.c
* Version 2.50
*
* Description:
*  This file provides all Interrupt Service functionality of the UART component
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "MODEM.h"
#include "cyapicallbacks.h"


/***************************************
* Custom Declarations
***************************************/
/* `#START CUSTOM_DECLARATIONS` Place your declaration here */

/* `#END` */

#if (MODEM_RX_INTERRUPT_ENABLED && (MODEM_RX_ENABLED || MODEM_HD_ENABLED))
    /*******************************************************************************
    * Function Name: MODEM_RXISR
    ********************************************************************************
    *
    * Summary:
    *  Interrupt Service Routine for RX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  MODEM_rxBuffer - RAM buffer pointer for save received data.
    *  MODEM_rxBufferWrite - cyclic index for write to rxBuffer,
    *     increments after each byte saved to buffer.
    *  MODEM_rxBufferRead - cyclic index for read from rxBuffer,
    *     checked to detect overflow condition.
    *  MODEM_rxBufferOverflow - software overflow flag. Set to one
    *     when MODEM_rxBufferWrite index overtakes
    *     MODEM_rxBufferRead index.
    *  MODEM_rxBufferLoopDetect - additional variable to detect overflow.
    *     Set to one when MODEM_rxBufferWrite is equal to
    *    MODEM_rxBufferRead
    *  MODEM_rxAddressMode - this variable contains the Address mode,
    *     selected in customizer or set by UART_SetRxAddressMode() API.
    *  MODEM_rxAddressDetected - set to 1 when correct address received,
    *     and analysed to store following addressed data bytes to the buffer.
    *     When not correct address received, set to 0 to skip following data bytes.
    *
    *******************************************************************************/
    CY_ISR(MODEM_RXISR)
    {
        uint8 readData;
        uint8 readStatus;
        uint8 increment_pointer = 0u;

    #if(CY_PSOC3)
        uint8 int_en;
    #endif /* (CY_PSOC3) */

    #ifdef MODEM_RXISR_ENTRY_CALLBACK
        MODEM_RXISR_EntryCallback();
    #endif /* MODEM_RXISR_ENTRY_CALLBACK */

        /* User code required at start of ISR */
        /* `#START MODEM_RXISR_START` */

        /* `#END` */

    #if(CY_PSOC3)   /* Make sure nested interrupt is enabled */
        int_en = EA;
        CyGlobalIntEnable;
    #endif /* (CY_PSOC3) */

        do
        {
            /* Read receiver status register */
            readStatus = MODEM_RXSTATUS_REG;
            /* Copy the same status to readData variable for backward compatibility support 
            *  of the user code in MODEM_RXISR_ERROR` section. 
            */
            readData = readStatus;

            if((readStatus & (MODEM_RX_STS_BREAK | 
                            MODEM_RX_STS_PAR_ERROR |
                            MODEM_RX_STS_STOP_ERROR | 
                            MODEM_RX_STS_OVERRUN)) != 0u)
            {
                /* ERROR handling. */
                MODEM_errorStatus |= readStatus & ( MODEM_RX_STS_BREAK | 
                                                            MODEM_RX_STS_PAR_ERROR | 
                                                            MODEM_RX_STS_STOP_ERROR | 
                                                            MODEM_RX_STS_OVERRUN);
                /* `#START MODEM_RXISR_ERROR` */

                /* `#END` */
                
            #ifdef MODEM_RXISR_ERROR_CALLBACK
                MODEM_RXISR_ERROR_Callback();
            #endif /* MODEM_RXISR_ERROR_CALLBACK */
            }
            
            if((readStatus & MODEM_RX_STS_FIFO_NOTEMPTY) != 0u)
            {
                /* Read data from the RX data register */
                readData = MODEM_RXDATA_REG;
            #if (MODEM_RXHW_ADDRESS_ENABLED)
                if(MODEM_rxAddressMode == (uint8)MODEM__B_UART__AM_SW_DETECT_TO_BUFFER)
                {
                    if((readStatus & MODEM_RX_STS_MRKSPC) != 0u)
                    {
                        if ((readStatus & MODEM_RX_STS_ADDR_MATCH) != 0u)
                        {
                            MODEM_rxAddressDetected = 1u;
                        }
                        else
                        {
                            MODEM_rxAddressDetected = 0u;
                        }
                    }
                    if(MODEM_rxAddressDetected != 0u)
                    {   /* Store only addressed data */
                        MODEM_rxBuffer[MODEM_rxBufferWrite] = readData;
                        increment_pointer = 1u;
                    }
                }
                else /* Without software addressing */
                {
                    MODEM_rxBuffer[MODEM_rxBufferWrite] = readData;
                    increment_pointer = 1u;
                }
            #else  /* Without addressing */
                MODEM_rxBuffer[MODEM_rxBufferWrite] = readData;
                increment_pointer = 1u;
            #endif /* (MODEM_RXHW_ADDRESS_ENABLED) */

                /* Do not increment buffer pointer when skip not addressed data */
                if(increment_pointer != 0u)
                {
                    if(MODEM_rxBufferLoopDetect != 0u)
                    {   /* Set Software Buffer status Overflow */
                        MODEM_rxBufferOverflow = 1u;
                    }
                    /* Set next pointer. */
                    MODEM_rxBufferWrite++;

                    /* Check pointer for a loop condition */
                    if(MODEM_rxBufferWrite >= MODEM_RX_BUFFER_SIZE)
                    {
                        MODEM_rxBufferWrite = 0u;
                    }

                    /* Detect pre-overload condition and set flag */
                    if(MODEM_rxBufferWrite == MODEM_rxBufferRead)
                    {
                        MODEM_rxBufferLoopDetect = 1u;
                        /* When Hardware Flow Control selected */
                        #if (MODEM_FLOW_CONTROL != 0u)
                            /* Disable RX interrupt mask, it is enabled when user read data from the buffer using APIs */
                            MODEM_RXSTATUS_MASK_REG  &= (uint8)~MODEM_RX_STS_FIFO_NOTEMPTY;
                            CyIntClearPending(MODEM_RX_VECT_NUM);
                            break; /* Break the reading of the FIFO loop, leave the data there for generating RTS signal */
                        #endif /* (MODEM_FLOW_CONTROL != 0u) */
                    }
                }
            }
        }while((readStatus & MODEM_RX_STS_FIFO_NOTEMPTY) != 0u);

        /* User code required at end of ISR (Optional) */
        /* `#START MODEM_RXISR_END` */

        /* `#END` */

    #ifdef MODEM_RXISR_EXIT_CALLBACK
        MODEM_RXISR_ExitCallback();
    #endif /* MODEM_RXISR_EXIT_CALLBACK */

    #if(CY_PSOC3)
        EA = int_en;
    #endif /* (CY_PSOC3) */
    }
    
#endif /* (MODEM_RX_INTERRUPT_ENABLED && (MODEM_RX_ENABLED || MODEM_HD_ENABLED)) */


#if (MODEM_TX_INTERRUPT_ENABLED && MODEM_TX_ENABLED)
    /*******************************************************************************
    * Function Name: MODEM_TXISR
    ********************************************************************************
    *
    * Summary:
    * Interrupt Service Routine for the TX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  MODEM_txBuffer - RAM buffer pointer for transmit data from.
    *  MODEM_txBufferRead - cyclic index for read and transmit data
    *     from txBuffer, increments after each transmitted byte.
    *  MODEM_rxBufferWrite - cyclic index for write to txBuffer,
    *     checked to detect available for transmission bytes.
    *
    *******************************************************************************/
    CY_ISR(MODEM_TXISR)
    {
    #if(CY_PSOC3)
        uint8 int_en;
    #endif /* (CY_PSOC3) */

    #ifdef MODEM_TXISR_ENTRY_CALLBACK
        MODEM_TXISR_EntryCallback();
    #endif /* MODEM_TXISR_ENTRY_CALLBACK */

        /* User code required at start of ISR */
        /* `#START MODEM_TXISR_START` */

        /* `#END` */

    #if(CY_PSOC3)   /* Make sure nested interrupt is enabled */
        int_en = EA;
        CyGlobalIntEnable;
    #endif /* (CY_PSOC3) */

        while((MODEM_txBufferRead != MODEM_txBufferWrite) &&
             ((MODEM_TXSTATUS_REG & MODEM_TX_STS_FIFO_FULL) == 0u))
        {
            /* Check pointer wrap around */
            if(MODEM_txBufferRead >= MODEM_TX_BUFFER_SIZE)
            {
                MODEM_txBufferRead = 0u;
            }

            MODEM_TXDATA_REG = MODEM_txBuffer[MODEM_txBufferRead];

            /* Set next pointer */
            MODEM_txBufferRead++;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START MODEM_TXISR_END` */

        /* `#END` */

    #ifdef MODEM_TXISR_EXIT_CALLBACK
        MODEM_TXISR_ExitCallback();
    #endif /* MODEM_TXISR_EXIT_CALLBACK */

    #if(CY_PSOC3)
        EA = int_en;
    #endif /* (CY_PSOC3) */
   }
#endif /* (MODEM_TX_INTERRUPT_ENABLED && MODEM_TX_ENABLED) */


/* [] END OF FILE */
