/*******************************************************************************
* File Name: MODEM_PM.c
* Version 2.50
*
* Description:
*  This file provides Sleep/WakeUp APIs functionality.
*
* Note:
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "MODEM.h"


/***************************************
* Local data allocation
***************************************/

static MODEM_BACKUP_STRUCT  MODEM_backup =
{
    /* enableState - disabled */
    0u,
};



/*******************************************************************************
* Function Name: MODEM_SaveConfig
********************************************************************************
*
* Summary:
*  This function saves the component nonretention control register.
*  Does not save the FIFO which is a set of nonretention registers.
*  This function is called by the MODEM_Sleep() function.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  MODEM_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void MODEM_SaveConfig(void)
{
    #if(MODEM_CONTROL_REG_REMOVED == 0u)
        MODEM_backup.cr = MODEM_CONTROL_REG;
    #endif /* End MODEM_CONTROL_REG_REMOVED */
}


/*******************************************************************************
* Function Name: MODEM_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the nonretention control register except FIFO.
*  Does not restore the FIFO which is a set of nonretention registers.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  MODEM_backup - used when non-retention registers are restored.
*
* Reentrant:
*  No.
*
* Notes:
*  If this function is called without calling MODEM_SaveConfig() 
*  first, the data loaded may be incorrect.
*
*******************************************************************************/
void MODEM_RestoreConfig(void)
{
    #if(MODEM_CONTROL_REG_REMOVED == 0u)
        MODEM_CONTROL_REG = MODEM_backup.cr;
    #endif /* End MODEM_CONTROL_REG_REMOVED */
}


/*******************************************************************************
* Function Name: MODEM_Sleep
********************************************************************************
*
* Summary:
*  This is the preferred API to prepare the component for sleep. 
*  The MODEM_Sleep() API saves the current component state. Then it
*  calls the MODEM_Stop() function and calls 
*  MODEM_SaveConfig() to save the hardware configuration.
*  Call the MODEM_Sleep() function before calling the CyPmSleep() 
*  or the CyPmHibernate() function. 
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  MODEM_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void MODEM_Sleep(void)
{
    #if(MODEM_RX_ENABLED || MODEM_HD_ENABLED)
        if((MODEM_RXSTATUS_ACTL_REG  & MODEM_INT_ENABLE) != 0u)
        {
            MODEM_backup.enableState = 1u;
        }
        else
        {
            MODEM_backup.enableState = 0u;
        }
    #else
        if((MODEM_TXSTATUS_ACTL_REG  & MODEM_INT_ENABLE) !=0u)
        {
            MODEM_backup.enableState = 1u;
        }
        else
        {
            MODEM_backup.enableState = 0u;
        }
    #endif /* End MODEM_RX_ENABLED || MODEM_HD_ENABLED*/

    MODEM_Stop();
    MODEM_SaveConfig();
}


/*******************************************************************************
* Function Name: MODEM_Wakeup
********************************************************************************
*
* Summary:
*  This is the preferred API to restore the component to the state when 
*  MODEM_Sleep() was called. The MODEM_Wakeup() function
*  calls the MODEM_RestoreConfig() function to restore the 
*  configuration. If the component was enabled before the 
*  MODEM_Sleep() function was called, the MODEM_Wakeup()
*  function will also re-enable the component.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  MODEM_backup - used when non-retention registers are restored.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void MODEM_Wakeup(void)
{
    MODEM_RestoreConfig();
    #if( (MODEM_RX_ENABLED) || (MODEM_HD_ENABLED) )
        MODEM_ClearRxBuffer();
    #endif /* End (MODEM_RX_ENABLED) || (MODEM_HD_ENABLED) */
    #if(MODEM_TX_ENABLED || MODEM_HD_ENABLED)
        MODEM_ClearTxBuffer();
    #endif /* End MODEM_TX_ENABLED || MODEM_HD_ENABLED */

    if(MODEM_backup.enableState != 0u)
    {
        MODEM_Enable();
    }
}


/* [] END OF FILE */
